#pragma once

#include "value.h"

//! A range is defined by two border values, Inf and Sup.
class Range {
 private:
    enum State { UNDEF, EMPTY, FINITE } state;
    Value Inf;
    Value Sup;

 public:
    Range() : Inf(+1), Sup(-1), state(UNDEF) {};
    Range( const Value &inf_in, const Value &sup_in ) :
        Inf(inf_in), Sup(sup_in)
        { state = inf_in > sup_in ? EMPTY : FINITE ; };
    
    Value inf() const { 
        if( state!=FINITE ) throw ProgramError( "Range inf()" );
        else return Inf; };
    Value sup() const { 
        if( state!=FINITE ) throw ProgramError( "Range sup()" );
        else return Sup; };
    
    string strg() const {
        if     ( state==UNDEF ) return "";
        else if( state==EMPTY ) return "<empty range>";
        else                    return Inf.strg() + " .. " + Sup.strg();
    };
    bool is_defined() const { return state!=UNDEF; };

    bool contains( const Value &v ) const {
        if( state==UNDEF || state==EMPTY) return false;
        else return Inf<=v && v<=Sup;
    };
    // quite specific for use in plot:
    double relval( int logflag, double v ) const {
        if ( state!=FINITE ) return 0;
        if (logflag) {
            if(Inf.d()<0 || v<0) return 0;
            return  log(v/Inf.d()) / log(Sup.d()/Inf.d());
	} else {
            return (v-Inf.d()) / (Sup.d()-Inf.d());
	}
    };
};
